#!/bin/bash

test -r /opt/RNDIS/conf.ini && . /opt/RNDIS/conf.ini
[ "$RNDISSTART" -ne "1" ] && exit 0

log_file=/linuxcash/logs/current/rndis.log
log () {
    log_dir=`dirname $log_file`
    [ -d $log_dir ] || mkdir -p $log_dir
    while read data; do
        echo "$(date +"%F %X.%3N") - $data" >> $log_file
    done
}
exec > >(log) 2>&1

IFACE_MAC=$1
echo "Подключен интерфейс $INTERFACE, MAC=$IFACE_MAC"

# Поиск привязок к маку
# Ищутся все параметры вида MACN, где N это номер
# Если MAC интерфейса совпадает с одним из найденных параметров MACN
# тогда будут использоваться настройки с номером N
for MAC in $(compgen -A variable | grep '^MAC[0-9]$'); do
    if [ "${!MAC}" = "$IFACE_MAC" ] ; then
        DEVNUM=${MAC#MAC}
        TMPADDR="ADDRESS$DEVNUM"
        ADDRESS=${!TMPADDR}
        TMPMASK="MASK$DEVNUM"
        MASK=${!TMPMASK}
        break
    fi
done

if [ "$ADDRESS" = "" ] || [ "$MASK" = "" ] ; then
    echo "В настройках не заданы адрес и маска"
    exit 0
fi

echo "Установка соединения, интерфейс $INTERFACE, адрес $ADDRESS, маска $MASK"

fail() {
    echo "Ошибка установки соединения"
    exit 0
}

sysctl -w net.ipv4.ip_forward=1 || fail
iptables -t nat -D POSTROUTING -s $ADDRESS/$MASK -d 0/0 -j MASQUERADE
iptables -t nat -I POSTROUTING -s $ADDRESS/$MASK -d 0/0 -j MASQUERADE || fail
ifconfig $INTERFACE $ADDRESS || fail
echo "Соединение успешно установлено"
