#!/bin/bash

log_file=/linuxcash/logs/current/rndis.log
log () {
    log_dir=`dirname $log_file`
    [ -d $log_dir ] || mkdir -p $log_dir
    while read data; do
        echo "$(date +"%F %X.%3N") - $data" >> $log_file
    done
}
exec > >(log) 2>&1

echo "Старт сервиса проверки состояния интерфейсов RNDIS"

findNetDevices () {
    # Ищем сетевые устройства
    pushd /sys/bus/usb/devices
    # Список устройств, ex: (1-2 1-3)
    devs=()
    # Проходим по symlink usb-устройств
    while read -r -d $'\0'; do
    # Работаем только с ссылками формата 1-2, для нормального переподключения (bind/unbind)
    if echo "$REPLY" | grep -oqP '[0-9]+\-[0-9]+(\.[0-9]+)*' ; then
        # Читаем путь до директории устройства из symlink
        LINK=$(readlink $REPLY);
        # Для RNDIS устройств семейства Штрих-М достаточно проверить атрибут product
        if [ -f $LINK/product ] && grep -q RNDIS $LINK/product; then
            devs+=(${REPLY#./})
        # У ФР POScenter атрибут product бесполезный, потому смотрим устройства с vid=0d28
        elif [ -f $LINK/idVendor ] && grep -qE '0d28' $LINK/idVendor; then
            # Проверяем, что устройство использует драйвер cdc_ncm
            # В директории устройства ищем symlink до используемого драйвера, читаем их и проверяем
            # наличие драйвера cdc_ncm или rndis_host
            if 2>/dev/null grep -qE '(cdc_ncm|rndis)' < <(find $LINK -maxdepth 2 -type l -name driver -exec readlink {} \;); then
                devs+=(${REPLY#./})
            fi
        fi
    fi
    done < <(find -type l -print0)
    popd
}

while true; do
    echo "Проверка состояния интерфейсов RNDIS"
    findNetDevices
    cd /sys/bus/usb/drivers/usb/
    devs_count=${#devs[@]}

    # Кол-во сетевых интерфейсов RNDIS. Поднимает их /etc/udev/rules.d/99-shtrihrndis.rules
    iface_count=`ifconfig | grep -c RNDIS`
    echo "Количество RNDIS устройств $devs_count, интерфейсов $iface_count"

    # Если количество usb-net-устройств меньше поднятых сетевых интерфейсов, то переподключаем устройства,
    # остальное сделает udev
    if [ "$devs_count" -gt "$iface_count" ]; then
        for dev in ${devs[@]}; do
            echo "Переподключение устройства /sys/bus/usb/drivers/usb/$dev"
            echo $dev > /sys/bus/usb/drivers/usb/unbind
            sleep 2
            echo $dev > /sys/bus/usb/drivers/usb/bind
        done
    fi

    echo "Проверка выполнена, следующая проверка через 10 минут"
    sleep 600
done
